@@ -1,13 +1,13 @@
 package Workflow::Persister;
 
-# $Id: Persister.pm 375 2008-04-22 19:54:42Z jonasbn $
+# $Id: Persister.pm,v 1.8 2004/05/14 05:13:52 cwinters Exp $
 
 use strict;
 use base qw( Workflow::Base );
 use Log::Log4perl       qw( get_logger );
 use Workflow::Exception qw( persist_error );
 
-$Workflow::Persister::VERSION = '1.09';
+$Workflow::Persister::VERSION  = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
 
 my @FIELDS = qw( name class
                  use_random use_uuid
@@ -115,15 +115,6 @@ sub fetch_history {
                   "'fetch_history()'";
 }
 
-# Only required for DBI persisters.
-sub commit_transaction {
-    return;
-}
-
-sub rollback_transaction {
-  return;
-}
-
 1;
 
 __END__
@@ -166,98 +157,27 @@ fetches workflow history records.
 
 =head2 Methods
 
-=head3 create_workflow( $workflow )
-
-Stub that warns that the method should be overwritten in the derived
-Persister. Since this is a SUPER class.
+B<create_workflow( $workflow )>
 
 Generate an ID for the workflow, serialize the workflow data (ID and
 state) and set the ID in the workflow.
 
-=head3 update_workflow( $workflow )
-
-Stub that warns that the method should be overwritten in the derived
-Persister. Since this is a SUPER class.
+B<update_workflow( $workflow )>
 
 Update the workflow state.
 
-=head3 fetch_workflow( $workflow_id )
-
-Stub that warns that the method should be overwritten in the derived
-Persister. Since this is a SUPER class.
+B<fetch_workflow( $workflow_id )>
 
 Retrieve the workflow data corresponding to C<$workflow_id>. It not
 found return undef, if found return a hashref with the data.
 
-=head3 create_history( $workflow, @history )
-
-Stub that warns that the method should be overwritten in the derived
-Persister. Since this is a SUPER class.
+B<create_history( $workflow, @history )>
 
 Serialize all objects in C<@history> for later retrieval.
 
-=head3 fetch_history( $workflow )
+B<fetch_history( $workflow )>
 
-Stub that warns that the method should be overwritten in the derived
-Persister. Since this is a SUPER class.
-
-The derived class method should return a list of L<Workflow::History> objects.
-
-=head3 assign_generators( \%params ) 
-
-Assigns proper generators based on intialization, see L</init>
-
-=head3 fetch_extra_workflow_data ( $workflow )
-
-A stub that warns that the method should be overwritten in the derived
-Persister. Since this is a SUPER class.
-
-=head3 commit_transaction
-
-Commit the current transaction if the persister supports transactions.
-This stub does not have to be overridden. It is not executed if
-autocommit is on.
-
-=head3 rollback_transaction
-
-Roll back the current transaction if the persister supports transactions.
-This stub does not have to be overridden. It is not executed if
-autocommit is on.
-
-=head3 init
-
-Method to initialize persister based on configuration.
-
-=head3 init_random_generators( \%params )
-
-Initializes random id generators, takes the following named parameters:
-
-=over
-
-=item * length, of random id to be generated
-
-=back
-
-Returns two identical random id generator objects in list context.
-
-=head3 init_uuid_generators( \%params )
-
-Initializes UUID generators, takes no parameters
-
-Returns two identical UUID generator objects in list context.
-
-=head1 TODO
-
-=over
-
-=item * refactor init_random_generators, returns two similar objects?
-
-=item * refactor init_uuid_generators, returns two similar objects?
-
-=item * refactor init_uuid_generators, takes no parameters, even though
-we shift parameters in?
-
-=back
+Return list of L<Workflow::History> objects.
 
 =head1 SEE ALSO
 
@@ -267,15 +187,11 @@ L<Workflow::History>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2007 Chris Winters. All rights reserved.
+Copyright (c) 2003-2004 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =head1 AUTHORS
 
-Jonas B. Nielsen (jonasbn) E<lt>jonasbn@cpan.orgE<gt> is the current maintainer.
-
-Chris Winters E<lt>chris@cwinters.comE<gt>, original author.
-
-=cut
+Chris Winters E<lt>chris@cwinters.comE<gt>
