@@ -1,11 +1,11 @@
 package Workflow::Persister::UUID;
 
-# $Id: UUID.pm 299 2007-07-03 14:52:11Z jonasbn $
+# $Id: UUID.pm,v 1.2 2004/03/08 04:56:09 cwinters Exp $
 
 use strict;
 use Data::UUID;
 
-$Workflow::Persister::UUID::VERSION = '1.03';
+$Workflow::Persister::UUID::VERSION  = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
 
 sub new {
     my ( $class, $params ) = @_;
@@ -45,38 +45,6 @@ UUIDs/GUIDs generated until 3400 CE."
 This uses the L<Data::UUID> module to generate the UUID string, so
 look there if you are curious about the algorithm, efficiency, etc.
 
-=head2 METHODS
-
-=head3 new
-
-Instantiates a Workflow::Persister::UUID object, which is actually an
-encapsulation of L<Data::UUID>.
-
-=head3 pre_fetch_id
-
-L</pre_fetch_id> can then be used to generate/retrieve a unique ID, generated
-by L<Data::UUID>.
-
-=head3 post_fetch_id
-
-This method is unimplemented at this time, please see the TODO.
-
-=head1 TODO
-
-=over
-
-=item * Implement L</post_fetch_id>
-
-=back 
-
-=head1 SEE ALSO
-
-=over
-
-=item L<Data::UUID>
-
-=back
-
 =head1 COPYRIGHT
 
 Copyright (c) 2003-2004 Chris Winters. All rights reserved.
